@@ -33,50 +33,65 @@
 summarizeForROC <- function(test,thr=NULL,tyThr="BH",columnTest=1,spec=c("H","E","S"),annotCol="spec",tit=NULL,color=1,plotROC=TRUE,pch=1,bg=NULL,overlPlot=FALSE,silent=FALSE,callFrom=NULL) {
   ## summarize esting result by species (3rd is supposed as reference)
   argN <- deparse(substitute(test))
-  fxNa <- wrMisc::.composeCallName(callFrom,newNa="summarizeForROC")
+  fxNa <- wrMisc::.composeCallName(callFrom, newNa="summarizeForROC")
   chLst <- tyThr %in% names(test)
-  if(!any(chLst)) stop("Don't know what kind of test-results to use.  Can't find element '",tyThr[!chLst],"' in elements of 'test' !!")
+  if(any(!chLst)) stop("Don't know what kind of test-results to use.  Can't find element '",tyThr[!chLst],"' in elements of 'test' !!")
   if(!"annot" %in% names(test)) stop("test$annot is needed to map content of 'spec'")
-  if(is.null(thr)) thr <- signif(c(as.numeric(sapply((1:4)*2,function(x) x*c(1e-4,1e-5,1e-6,1e-7))),
+  if(is.null(thr)) thr <- signif(c(as.numeric(sapply((1:4)*2, function(x) x*c(1e-4,1e-5,1e-6,1e-7))),
     seq(0,1,length.out=50)^5,seq(0,1,length.out=50)^2,seq(0,1,length.out=61),4^(-2:-10),1.01),2)
   thr <- sort(unique(abs(wrMisc::naOmit(thr))))                                    # 151 -> 108 values for default
-  pp <- matrix(nrow=length(thr),ncol=4,dimnames=list(NULL,c("TP","FP","FN","TN")))
-  spiSpec <- if(ncol(test$annot) >1) {test$annot[,annotCol] %in% spec[-1]} else {test$annot[,1] %in% spec[-1]} 
+  pp <- matrix(nrow=length(thr), ncol=4, dimnames=list(NULL,c("TP","FP","FN","TN")))
+  chSpec <- spec %in% test$annot[,annotCol]
+  if(all(!chSpec)) stop(" None of the elements of argument 'spec' found in column '",annotCol,"' !!")
+  if(any(!chSpec)) message(fxNa," Trouble ahead ?  Elements ",wrMisc::pasteC(spec[which(!chSpec)],quoteC="'")," not found !!")
+  spiSpec <- if(ncol(test$annot) >1) {test$annot[,annotCol] %in% spec[-1]} else {test$annot[,1] %in% spec[-1]}
+  if(all(!spiSpec)) message(fxNa," Trouble ahead ?  Could not find any element annotated as species to search for (ie, TP will walways remain 0)")
   if(length(dim(test[[tyThr]])) ==2) if(ncol(test[[tyThr]])==2 & identical(colnames(test[[tyThr]])[1],"(Intercept)") & columnTest !=2) {
     message(fxNa," Value of argument 'columnTest' seems bizzare, setting to =2 !  (to avoid testing '(Intercept)')")
     columnTest <- 2 }
   tmp1 <- if(length(dim(test[[tyThr]])) ==2) test[[tyThr]][which(spiSpec),columnTest] else test[[tyThr]][which(spiSpec)]
   tmp2 <- if(length(dim(test[[tyThr]])) ==2) test[[tyThr]][which(!spiSpec),columnTest] else test[[tyThr]][which(!spiSpec)]
-  pp <- cbind(
-    TP= sapply(thr,function(x) sum(tmp1 <=x,na.rm=TRUE)),
-    FP= sapply(thr,function(x) sum(tmp2 <=x,na.rm=TRUE)),
-    FN= sapply(thr,function(x) sum(tmp1 >x,na.rm=TRUE)),
-    TN= sapply(thr,function(x) sum(tmp2 >x,na.rm=TRUE)))
-  keyVal <- cbind(alph=thr,
-    spec=as.numeric(pp[,"TN"]/(pp[,"TN"]+pp[,"FP"])), sens=as.numeric(c(pp[,"TP"]/(pp[,"TP"]+pp[,"FN"]))),
-    prec=as.numeric(pp[,"TP"]/(pp[,"FP"]+pp[,"TP"])), accur=as.numeric((pp[,"TP"]+pp[,"TN"])/rowSums(pp)), 
-    FDR=as.numeric(pp[,"FP"]/(pp[,"TP"]+pp[,"FP"])))
-  ## add no of lines/prot retained for each species
-  tmp3 <- test$annot[,annotCol] %in% spec[2]
-  tmp3 <- if(length(dim(test[[tyThr]])) >1) test[[tyThr]][which(tmp3),columnTest] else test[[tyThr]][which(tmp3)]       # pvalues for 1st spike-in species (eg E)
-  if(length(spec) >2) {
-    tmp4 <- test$annot[,annotCol] %in% spec[3]
-    tmp4 <- if(length(dim(test[[tyThr]])) >1) test[[tyThr]][which(tmp4),columnTest] else test[[tyThr]][which(tmp4)] }      # pvalues for 2nd spike-in species (eg S)
-  tmp3 <- cbind(Sp1Pos=pp[,"FP"], Sp2Pos=sapply(thr,function(x) sum(tmp3 <=x,na.rm=TRUE)),
-    Sp3Pos=if(length(spec) >2) sapply(thr,function(x) sum(tmp4 <=x,na.rm=TRUE)) else NULL)
-  colnames(tmp3) <- paste("n.pos",spec,sep=".")  
-  keyVal <- cbind(keyVal,tmp3)  
-  if(plotROC) {
-    if(is.null(tit)) tit <- paste("ROC of ",argN)
-    xLab <- "1 - Specificity"
-    yLab <- "Sensitivity"
-    if(overlPlot) graphics::points(1-keyVal[,annotCol],keyVal[,"sens"],col=color,pch=pch,bg=NULL,type="S") else {
-      graphics::plot(1-keyVal[,annotCol],keyVal[,"sens"],col=color,pch=pch,bg=bg,type="S",main=tit,xlab=xLab,ylab=yLab,xlim=c(0,1),ylim=c(0,1))}
-    cutP <- keyVal[which(keyVal[,1]==0.05),-1]
-    newPch <- cbind(c(1,16,2,17, 7,15,5,6),new=c(21,21,24,24,22,22,23,25))                                   # transform open or plain filled points to color-filled
-    if(pch %in% newPch[,1]){ pch2 <- newPch[which(newPch[,1]==pch),2]; bg <- color; col2 <- grDevices::grey(0.2)} else {pch2 <- pch; col2 <- color}   
-    col2 <- wrMisc::convColorToTransp(col2,alph=0.9)
-    graphics::points(1-cutP[annotCol],cutP["sens"],col=col2,pch=pch2,bg=bg,cex=1.4)  
-  }
+  if(all(is.na(tmp1))) {
+    message(fxNa," PROBLEM :\n  ***  None of the elements annotated as 'positive' species to search for has any valid testing results ! Unable to construct TP ! ***")
+    keyVal <- NULL
+  } else {
+   #cat("su1\n"); su1 <<- list(test=test,tyThr=tyThr,columnTest=columnTest,spec=spec,annotCol=annotCol, thr=thr,pp=pp,spiSpec=spiSpec,tmp1=tmp1,tmp2=tmp2)
+    pp <- cbind(
+      TP= sapply(thr,function(x) sum(tmp1 <=x,na.rm=TRUE)),
+      FP= sapply(thr,function(x) sum(tmp2 <=x,na.rm=TRUE)),
+      FN= sapply(thr,function(x) sum(tmp1 >x,na.rm=TRUE)),
+      TN= sapply(thr,function(x) sum(tmp2 >x,na.rm=TRUE)))
+  
+    keyVal <- cbind(alph=thr,
+      spec=as.numeric(pp[,"TN"]/(pp[,"TN"]+pp[,"FP"])), sens=as.numeric(c(pp[,"TP"]/(pp[,"TP"]+pp[,"FN"]))),
+      prec=as.numeric(pp[,"TP"]/(pp[,"FP"]+pp[,"TP"])), accur=as.numeric((pp[,"TP"]+pp[,"TN"])/rowSums(pp)), 
+      FDR=as.numeric(pp[,"FP"]/(pp[,"TP"]+pp[,"FP"])))
+    chNaN <- colSums(is.nan(keyVal)) ==nrow(keyVal)
+    if(any(chNaN)) keyVal[,which(chNaN)] <- 0
+    ## add no of lines/prot retained for each species
+    tmp3 <- test$annot[,annotCol] %in% spec[2]
+    tmp3 <- if(length(dim(test[[tyThr]])) >1) test[[tyThr]][which(tmp3),columnTest] else test[[tyThr]][which(tmp3)]       # pvalues for 1st spike-in species (eg E)
+  
+     #cat("su2\n"); su2 <<- list(test=test,thr=thr,pp=pp,keyVal=keyVal,spiSpec=spiSpec,tmp1=tmp1,tmp2=tmp2,tmp3=tmp3)
+  
+    if(length(spec) >2) {
+      tmp4 <- test$annot[,annotCol] %in% spec[3]
+      tmp4 <- if(length(dim(test[[tyThr]])) >1) test[[tyThr]][which(tmp4),columnTest] else test[[tyThr]][which(tmp4)] }      # pvalues for 2nd spike-in species (eg S)
+    tmp3 <- cbind(Sp1Pos=pp[,"FP"], Sp2Pos=sapply(thr,function(x) sum(tmp3 <=x,na.rm=TRUE)),
+      Sp3Pos=if(length(spec) >2) sapply(thr,function(x) sum(tmp4 <=x,na.rm=TRUE)) else NULL)
+    colnames(tmp3) <- paste("n.pos",spec,sep=".")  
+    keyVal <- cbind(keyVal,tmp3)  
+    if(plotROC) {
+      if(is.null(tit)) tit <- paste("ROC of ",argN)
+      xLab <- "1 - Specificity"
+      yLab <- "Sensitivity"
+      if(overlPlot) graphics::points(1-keyVal[,"spec"],keyVal[,"sens"],col=color,pch=pch,bg=NULL,type="S") else {
+        graphics::plot(1-keyVal[,"spec"],keyVal[,"sens"],col=color,pch=pch,bg=bg,type="S",main=tit,xlab=xLab,ylab=yLab,xlim=c(0,1),ylim=c(0,1))}
+      cutP <- keyVal[which(keyVal[,1]==0.05),-1]
+      newPch <- cbind(c(1,16,2,17, 7,15,5,6),new=c(21,21,24,24,22,22,23,25))                                   # transform open or plain filled points to color-filled
+      if(pch %in% newPch[,1]){ pch2 <- newPch[which(newPch[,1]==pch),2]; bg <- color; col2 <- grDevices::grey(0.2)} else {pch2 <- pch; col2 <- color}   
+      col2 <- wrMisc::convColorToTransp(col2,alph=0.9)
+      graphics::points(1-cutP["spec"],cutP["sens"],col=col2,pch=pch2,bg=bg,cex=1.4)  
+  } }
   keyVal }     
      
